pub struct Subst<F, VS = Vec<VarNo>, RS = Vec<F>> { /* private fields */ }Expand description
Substitution mapping variables to replacement functions, created from slices of functions
VS and RS are the storage types, and can, e.g., be Vecs or slices.
Implementations
Sourceimpl<F, VS: AsRef<[VarNo]>, RS: AsRef<[F]>> Subst<F, VS, RS>
impl<F, VS: AsRef<[VarNo]>, RS: AsRef<[F]>> Subst<F, VS, RS>
Sourcepub fn new(vars: VS, replacements: RS) -> Self
pub fn new(vars: VS, replacements: RS) -> Self
Create a new substitution to replace the i-th variable of vars by the
i-th function in replacement
All variables of vars should be distinct. Furthermore, variables must
be handles for the respective decision diagram levels, e.g., the
respective Boolean function for B(C)DDs, and a singleton set for ZBDDs.
Panics if vars and replacements have different length