IsFloatingPoint

Trait IsFloatingPoint 

pub trait IsFloatingPoint {
    const FLOATING_POINT: bool;
    const MIN_EXP: i32;
}
Is the underlying type a floating point number?

Required Associated Constants

const FLOATING_POINT: bool

true iff the underlying type is a floating point number

const MIN_EXP: i32

One greater than the minimum possible normal power of 2 exponent, see f64::MIN_EXP for instance. 0 for integers

Dyn Compatibility

This trait is not dyn compatible.

In older versions of Rust, dyn compatibility was called "object safety".

Implementors

Source

impl IsFloatingPoint for F64

cbindgen:ignore

Source

const FLOATING_POINT: bool = true

Source

const MIN_EXP: i32 = f64::MIN_EXP

Source

impl IsFloatingPoint for Natural

cbindgen:ignore

Source

impl<T: ShlAssign<i32>> IsFloatingPoint for T

cbindgen:ignore

Source

impl<T> IsFloatingPoint for Saturating<T>

cbindgen:ignore