pub struct AbortOnDrop<'a>(pub &'a str);Expand description
Zero-sized struct that calls std::process::abort() if dropped
This is useful to make code exception safe. If there is a region that must not panic for safety reasons, you can use this to prevent further unwinding, at least.
Before aborting, the provided string is printed. If the guarded code in the
example panics, FATAL: Foo panicked. Aborting. will be printed to stderr.
Example
let panic_guard = AbortOnDrop("Foo panicked.");
// ... code that might panic ...
panic_guard.defuse();Tuple Fields
0: &'a strImplementations
Sourceimpl AbortOnDrop<'_>
impl AbortOnDrop<'_>