pub trait IsFloatingPoint {
const FLOATING_POINT: bool;
const MIN_EXP: i32;
}Expand description
Is the underlying type a floating point number?
Required Associated Constants
Sourceconst FLOATING_POINT: bool
const FLOATING_POINT: bool
true iff the underlying type is a floating point number
Sourceconst MIN_EXP: i32
const MIN_EXP: i32
One greater than the minimum possible normal power of 2 exponent, see
f64::MIN_EXP for instance. 0 for integers
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety".
Implementors
Sourceimpl IsFloatingPoint for F64
cbindgen:ignore
impl IsFloatingPoint for F64
cbindgen:ignore
Sourceimpl IsFloatingPoint for Natural
cbindgen:ignore
impl IsFloatingPoint for Natural
cbindgen:ignore
Sourceimpl<T: ShlAssign<i32>> IsFloatingPoint for T
cbindgen:ignore
impl<T: ShlAssign<i32>> IsFloatingPoint for T
cbindgen:ignore
Sourceimpl<T> IsFloatingPoint for Saturating<T>
cbindgen:ignore
impl<T> IsFloatingPoint for Saturating<T>
cbindgen:ignore